@@ -20,7 +20,7 @@ lavCor <- function(object,
                    # other options (for lavaan)
                    ...,
                    cor.smooth = FALSE,
-                   cor.smooth.tol = 1e-06,
+                   cor.smooth.tol = 1e-04, # was 1e-06 in <0.6-14
                    output = "cor") {
 
     # shortcut if object = lavaan object
@@ -47,11 +47,38 @@ lavCor <- function(object,
     # check object class
     if(inherits(object, "lavData")) {
         lav.data <- object
-    } else if(inherits(object, "data.frame")) {
+    } else if(inherits(object, "data.frame") ||
+              inherits(object, "matrix")) {
+        object <- as.data.frame(object)
         NAMES <- names(object)
         if(!is.null(group)) {
             NAMES <- NAMES[- match(group, NAMES)]
         }
+        if(is.logical(ordered)) {
+            ordered.flag <- ordered
+            if(ordered.flag) {
+                ordered <- NAMES
+                if(length(ov.names.x) > 0L) {
+                    ordered <- ordered[- which(ordered %in% ov.names.x) ]
+                }
+            } else {
+                ordered <- character(0L)
+            }
+        } else if(is.null(ordered)) {
+            ordered <- character(0L)
+        } else if(!is.character(ordered)) {
+            stop("lavaan ERROR: ordered argument must be a character vector")
+        } else if(length(ordered) == 1L && nchar(ordered) == 0L) {
+            ordered <- character(0L)
+        } else {
+            # check if all names in "ordered" occur in the dataset?
+            missing.idx <- which(!ordered %in% NAMES)
+            if(length(missing.idx) > 0L) { # FIXme: warn = FALSE has no eff
+                warning("lavaan WARNING: ordered variable(s): ",
+                     paste(ordered[missing.idx], collapse = " "),
+                     "\n  could not be found in the data and will be ignored")
+            }
+        }
         lav.data <- lavData(data = object, group = group,
                             ov.names = NAMES, ordered = ordered,
                             ov.names.x = ov.names.x,
@@ -80,7 +107,7 @@ lavCor <- function(object,
     if(!is.null(dots$meanstructure)) {
         meanstructure <- dots$meanstructure
     }
-    if(categorical || tolower(missing) %in% c("ml", "fiml", "direct")) {
+    if(lav.data@ngroups > 1L || categorical || tolower(missing) %in% c("ml", "fiml", "direct")) {
         meanstructure <- TRUE
     }
     if(!is.null(dots$fixed.x)) {
@@ -108,6 +135,8 @@ lavCor <- function(object,
                                                        conditional.x = conditional.x,
                                                        group.w.free = FALSE,
                                                        missing = missing,
+                                                       correlation = categorical,
+                                                       estimator = estimator,
                                                        mimic = mimic),
                                   sample.cov    = NULL,
                                   sample.mean   = NULL,
