@@ -387,6 +387,14 @@ lav_matrix_vech_match_idx <- function(n = 1L, diagonal = TRUE,
 # default dup:
 lav_matrix_duplication <- .dup3
 
+# duplication matrix for correlation matrices:
+# - it returns a matrix of size p^2 * (p*(p-1))/2
+# - the columns corresponding to the diagonal elements have been removed
+lav_matrix_duplication_cor <- function(n = 1L) {
+    out <- lav_matrix_duplication(n = n)
+    diag.idx <- lav_matrix_diagh_idx(n = n)
+    out[,-diag.idx,drop = FALSE]
+}
 
 # compute t(D) %*% A (without explicitly computing D)
 # sqrt(nrow(A)) is an integer
@@ -432,6 +440,30 @@ lav_matrix_duplication_dup_pre2 <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute t(D) %*% A (without explicitly computing D)
+# sqrt(nrow(A)) is an integer
+# A is not symmetric, and not even square, only n^2 ROWS
+# correlation version: ignoring diagonal elements
+lav_matrix_duplication_cor_pre <- function(A = matrix(0,0,0)) {
+
+    # number of rows
+    n2 <- NROW(A)
+
+    # square nrow(A) only, n2 = n^2
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[idx1, , drop = FALSE] + A[idx2 , , drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[u,] <- OUT[u,] / 2.0
+
+    OUT
+}
 
 # compute A %*% D (without explicitly computing D)
 # sqrt(ncol(A)) must be an integer
@@ -456,6 +488,32 @@ lav_matrix_duplication_post <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute A %*% D (without explicitly computing D)
+# sqrt(ncol(A)) must be an integer
+# A is not symmetric, and not even square, only n^2 COLUMNS
+# correlation version: ignoring the diagonal elements
+lav_matrix_duplication_cor_post <- function(A = matrix(0,0,0)) {
+
+    # number of columns
+    n2 <- NCOL(A)
+
+    # square A only, n2 = n^2
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[, idx1, drop = FALSE] + A[, idx2, drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[,u] <- OUT[,u] / 2.0
+
+    OUT
+}
+
+
 # compute t(D) %*% A %*% D (without explicitly computing D)
 # A must be a square matrix and sqrt(ncol) an integer
 lav_matrix_duplication_pre_post <- function(A = matrix(0,0,0)) {
@@ -480,6 +538,32 @@ lav_matrix_duplication_pre_post <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute t(D) %*% A %*% D (without explicitly computing D)
+# A must be a square matrix and sqrt(ncol) an integer
+# correlation version: ignoring diagonal elements
+lav_matrix_duplication_cor_pre_post <- function(A = matrix(0,0,0)) {
+
+    # number of columns
+    n2 <- NCOL(A)
+
+    # square A only, n2 = n^2
+    stopifnot(NROW(A) == n2, sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[idx1, , drop = FALSE] + A[idx2, , drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[u,] <- OUT[u,] / 2.0
+    OUT <- OUT[, idx1, drop = FALSE] + OUT[, idx2, drop = FALSE]
+    OUT[,u] <- OUT[,u] / 2.0
+
+    OUT
+}
+
 # create the generalized inverse of the duplication matrix (D^+_n):
 # it removes the duplicated elements in vec(S) to create vech(S)
 #
@@ -1074,7 +1158,7 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
         if(zero.warn) {
             warning("lavaan WARNING: matrix to be inverted contains zero cols/rows")
         }
-        S <- S[-zero.idx, -zero.idx]
+        S <- S[-zero.idx, -zero.idx, drop = FALSE]
     }
 
     P <- NCOL(S)
@@ -1089,16 +1173,28 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
         tmp <- S[1,1]
         S.inv <- matrix(1/tmp, 1, 1)
         if(logdet) {
-            attr(S.inv, "logdet") <- log(tmp)
+            if(tmp > 0) {
+                 attr(S.inv, "logdet") <- log(tmp)
+            } else {
+                 attr(S.inv, "logdet") <- -Inf
+            }
         }
     } else if(P == 2L) {
         a11 <- S[1,1]; a12 <- S[1,2]; a21 <- S[2,1]; a22 <- S[2,2]
         tmp <- a11*a22 - a12*a21
         if(tmp == 0) {
+            S.inv <- matrix(c(Inf, Inf, Inf, Inf), 2, 2)
+            if(logdet) {
+                attr(S.inv, "logdet") <- -Inf
+            }
         } else {
             S.inv <- matrix(c(a22/tmp, -a21/tmp, -a12/tmp, a11/tmp), 2, 2)
             if(logdet) {
-                attr(S.inv, "logdet") <- log(tmp)
+                if(tmp > 0) {
+                    attr(S.inv, "logdet") <- log(tmp)
+                } else {
+                     attr(S.inv, "logdet") <- -Inf
+                }
             }
         }
     } else if(Sinv.method == "eigen") {
