@@ -2,6 +2,7 @@
 # 1. the independence model
 # 2. the unrestricted model
 # 3. model + extra parameters (for modindices/lavTestScore)
+# 4. catML fit based on DWLS fit (for robust RMSEA/CFI)
 
 
 # 1. fit an 'independence' model
@@ -56,6 +57,7 @@ lav_object_independence <- function(object         = NULL,
     # new in 0.6-6: add lower bounds for ov.var
     if(!is.null(lavoptions$optim.bounds)) {
         lavoptions$bounds <- "doe.maar"
+        lavoptions$effect.coding <- "" # to avoid warning
         lavoptions$optim.bounds <- list(lower = "ov.var")
         lavpartable <- lav_partable_add_bounds(partable = lavpartable,
             lavpta = lavpta, lavh1 = lavh1, lavdata = lavdata,
@@ -107,9 +109,16 @@ lav_object_independence <- function(object         = NULL,
     lavoptions$check.vcov <- FALSE
     lavoptions$optim.bounds <- list() # we already have the bounds
 
-    # ALWAYS do.fit and set optim.method = "nlminb"
-    lavoptions$do.fit  <- TRUE
-    lavoptions$optim.method <- "nlminb"
+    # ALWAYS do.fit and set optim.method = "nlminb" (if npar > 0)
+    npar <- lav_partable_npar(lavpartable)
+    if(npar > 0L) {
+        lavoptions$do.fit  <- TRUE
+        lavoptions$optim.method <- "nlminb"
+    } else {
+        # perhaps a correlation structure?
+        lavoptions$optim.method = "none"
+        lavoptions$optim.force.converged <- TRUE
+    }
 
     # verbose?
     lavoptions$verbose <- verbose
@@ -367,3 +376,105 @@ lav_object_extended <- function(object, add = NULL,
 
     FIT
 }
+
+# 4. catml model
+lav_object_catml <- function(lavobject = NULL) {
+
+    stopifnot(inherits(lavobject, "lavaan"))
+    stopifnot(lavobject@Model@categorical)
+
+    # extract slots
+    lavdata <- lavobject@Data
+    lavsamplestats <- lavobject@SampleStats
+    lavoptions <- lavobject@Options
+    lavpta <- lavobject@pta
+
+    # if only categorical variables: remove thresholds and intercepts
+    refit <- FALSE
+    if(all(lavdata@ov$type == "ordered")) {
+        partable.catml <- parTable(lavobject)
+        rm.idx <- which(partable.catml$op %in% c("|", "~1"))
+        partable.catml <- partable.catml[-rm.idx,]
+        partable.catml <- lav_partable_complete(partable.catml)
+    } else {
+        refit <- TRUE
+        partable.catml <- parTable(lavobject)
+        partable.catml$start <- partable.catml$est
+        partable.catml$se <- NULL
+        rm.idx <- which(partable.catml$op %in% c("|", "~1"))
+        partable.catml <- partable.catml[-rm.idx,]
+        partable.catml$ustart <- partable.catml$est
+        for(b in seq_len(lavpta$nblocks)) {
+            ov.names.num <- lavpta$vnames$ov.num[[b]]
+            ov.var.idx <- which(partable.catml$op == "~~" &
+                                partable.catml$lhs %in% ov.names.num &
+                                partable.catml$lhs == partable.catml$rhs)
+            partable.catml$free[ov.var.idx] <- 0L
+        }
+        partable.catml <- lav_partable_complete(partable.catml)
+    }
+
+    # adapt lavsamplestats
+    for(g in seq_len(lavdata@ngroups)) {
+        lavsamplestats@WLS.V[[g]] <- NULL
+        lavsamplestats@WLS.VD[[g]] <- NULL
+        COR <- lavsamplestats@cov[[g]]
+        # check if COV is pd or not
+        ev <- eigen(COR, symmetric = TRUE, only.values = TRUE)$values
+        if(any(ev < .Machine$double.eps^(1/2))) {
+            # not PD!
+            COV <- cov2cor(lav_matrix_symmetric_force_pd(COR, tol = 1e-04))
+            lavsamplestats@cov[[g]] <- COV
+            lavsamplestats@var[[g]] <- diag(COV)
+            refit <- TRUE
+        } else {
+            COV <- COR
+        }
+
+        out <- lav_samplestats_icov(COV = COV, ridge = 1e-05,
+                                    x.idx = lavsamplestats@x.idx[[g]],
+                                    ngroups = lavdata@ngroups, g = g,
+                                    warn = FALSE)
+        lavsamplestats@icov[[g]] <- out$icov
+        lavsamplestats@cov.log.det[[g]] <- out$cov.log.det
+
+        #NACOV <- lavsamplestats@NACOV[[g]]
+        #nvar   <- nrow(COV)
+        #ntotal <- nrow(NACOV)
+        #pstar <- nvar*(nvar-1)/2
+        #nocor <- ntotal - pstar
+        #if(length(nocor) > 0L) {
+        #    lavsamplestats@NACOV[[g]] <- NACOV[-seq_len(nocor),
+        #                                       -seq_len(nocor)]
+        #}
+    }
+
+    # adapt lavoptions
+    lavoptions$estimator <- "catML"
+    lavoptions$verbose <- FALSE
+    lavoptions$debug <- FALSE
+    lavoptions$.categorical <- FALSE
+    lavoptions$categorical <- FALSE
+    lavoptions$correlation <- TRUE
+    lavoptions$meanstructure <- FALSE
+    lavoptions$conditional.x <- FALSE # fixme
+    lavoptions$information <- c("expected", "expected")
+    lavoptions$h1.information <- c("structured", "structured") # unlike DWLS
+    lavoptions$se <- "none"
+    lavoptions$test <- "standard" # always for now
+    lavoptions$baseline <- TRUE
+    if(!refit) {
+        lavoptions$optim.method <- "none"
+        lavoptions$optim.force.converged <- TRUE
+    } else {
+        lavoptions$optim.gradient <- "numerical"
+    }
+
+    # dummy fit
+    FIT <- lavaan(slotParTable = partable.catml,
+                  slotSampleStats = lavsamplestats,
+                  slotData = lavdata,
+                  slotOptions = lavoptions)
+
+    FIT
+}
