@@ -21,8 +21,23 @@
 
 setMethod("show", "lavaan",
 function(object) {
+    # efa?
+    efa.flag <- object@Options$model.type == "efa"
+
     # show only basic information
-    lav_object_print_short_summary(object)
+    res <- lav_object_summary(object, fit.measures = FALSE,
+                                      estimates    = FALSE,
+                                      modindices   = FALSE,
+                                      efa          = efa.flag)
+    if(efa.flag) {
+        # print (standardized) loadings only
+        class(res) <- c("lavaan.efa", "list")
+        print(res)
+    } else {
+        # print lavaan header
+        print(res)
+    }
+    invisible(res)
 })
 
 setMethod("summary", "lavaan",
@@ -37,68 +52,37 @@ function(object, header       = TRUE,
                  cov.std      = TRUE,
                  rsquare      = FALSE,
                  std.nox      = FALSE,
+                 fm.args      = list(standard.test        = "default",
+                                     scaled.test          = "default",
+                                     rmsea.ci.level       = 0.90,
+                                     rmsea.h0.closefit    = 0.05,
+                                     rmsea.h0.notclosefit = 0.08,
+                                     cat.check.pd         = TRUE),
                  modindices   = FALSE,
-                 nd = 3L) {
+                 nd = 3L, cutoff = 0.3, dot.cutoff = 0.1) {
 
-    # return object
-    res <- list()
+    # efa?
+    efa.flag <- object@Options$model.type == "efa"
 
-    # this is to avoid partial matching of 'std' with std.nox
-    standardized <- std || standardized
+    res <- lav_object_summary(object = object, header = header,
+               fit.measures = fit.measures, estimates = estimates,
+               ci = ci, fmi = fmi, std = std, standardized = standardized,
+               remove.step1 = remove.step1, cov.std = cov.std,
+               rsquare = rsquare, std.nox = std.nox, efa = efa.flag,
+               fm.args = fm.args, modindices = modindices)
+    # res has class c("lavaan.summary", "list")
 
-    if(std.nox) {
-        standardized <- TRUE
+    # what about nd? only used if we actually print; save as attribute
+    attr(res, "nd") <- nd
+
+    # if efa, add cutoff and dot.cutoff, and change class
+    if(efa.flag) {
+        #class(res) <- c("lavaan.summary.efa", "list")
+        attr(res, "cutoff") <- cutoff
+        attr(res, "dot.cutoff") <- dot.cutoff
     }
 
-    # print the 'short' summary
-    if(header) {
-        lav_object_print_short_summary(object, nd = nd)
-    }
-
-    # only if requested, the fit measures
-    if(fit.measures) {
-        if(length(object@Options$test) == 1L && object@Options$test == "none") {
-            warning("lavaan WARNING: fit measures not available if test = \"none\"\n\n")
-        } else if(object@optim$npar > 0L && !object@optim$converged) {
-            warning("lavaan WARNING: fit measures not available if model did not converge\n\n")
-        } else {
-            FIT <- fitMeasures(object, fit.measures="default")
-            res$FIT = FIT
-            print.lavaan.fitMeasures( FIT, nd = nd, add.h0 = FALSE )
-        }
-    }
-
-    if(estimates) {
-        PE <- parameterEstimates(object, ci = ci, standardized = standardized,
-                                 rsquare = rsquare, fmi = fmi,
-                                 cov.std = cov.std,
-                                 remove.eq = FALSE, remove.system.eq = TRUE,
-                                 remove.ineq = FALSE, remove.def = FALSE,
-                                 remove.nonfree = FALSE,
-                                 remove.step1 = remove.step1,
-                                 #remove.nonfree.scales = TRUE,
-                                 output = "text",
-                                 header = TRUE)
-        if(standardized && std.nox) {
-            #PE$std.all <- PE$std.nox
-            PE$std.all <- NULL
-        }
-        print(PE, nd = nd)
-        res$PE <- as.data.frame(PE)
-    }
-
-    # modification indices?
-    if(modindices) {
-        cat("Modification Indices:\n\n")
-        MI <- modificationIndices(object, standardized=TRUE, cov.std = cov.std)
-        print( MI )
-        res$MI <- MI
-    }
-
-    # return something invisibly, just for those who want this...
-    # new in 0.6-4
-    invisible(res)
-
+    res
 })
 
 
@@ -264,6 +248,14 @@ standardizedSolution <-
         }
     }
 
+    # always remove 'da' rows (if any)
+    if(any(LIST$op == "da")) {
+        da.idx <- which(LIST$op == "da")
+        LIST <- LIST[-da.idx,,drop = FALSE]
+    }
+
+
+
     if(output == "text") {
         class(LIST) <- c("lavaan.parameterEstimates", "lavaan.data.frame",
                          "data.frame")
@@ -468,6 +460,11 @@ parameterestimates <- function(object,
        "bootstrap" %in%  object@Options$test ||
        "bollen.stine" %in% object@Options$test) {
         BOOT <- lav_object_inspect_boot(object)
+        bootstrap.seed <- attr(BOOT, "seed") # for bca
+        error.idx <- attr(BOOT, "error.idx")
+        if(length(error.idx) > 0L) {
+            BOOT <- BOOT[-error.idx,,drop = FALSE] # drops attributes
+        }
     } else {
         BOOT <- NULL
     }
@@ -507,10 +504,11 @@ parameterestimates <- function(object,
             }
 
             stopifnot(!is.null(BOOT))
-            stopifnot(boot.ci.type %in% c("norm","basic","perc","bca.simple"))
+            stopifnot(boot.ci.type %in% c("norm","basic","perc",
+                                          "bca.simple", "bca"))
             if(boot.ci.type == "norm") {
                 fac <- qnorm(a)
-                boot.x <- colMeans(BOOT)
+                boot.x <- colMeans(BOOT, na.rm = TRUE)
                 boot.est <-
                     lav_model_get_parameters(object@Model,
                                        GLIST=lav_model_x2GLIST(object@Model, boot.x),
@@ -612,6 +610,113 @@ parameterestimates <- function(object,
                    }
                }
 
+               # TODO:
+               # - add cin/ceq
+            } else if(boot.ci.type == "bca") { # new in 0.6-12
+               # we assume that the 'ordinary' (nonparametric) was used
+
+               lavoptions <- object@Options
+               ngroups <- object@Data@ngroups
+               nobs <- object@SampleStats@nobs
+               ntotal <- object@SampleStats@ntotal
+
+               # we need enough bootstrap runs
+               if(nrow(BOOT) < ntotal) {
+                   txt <- paste("BCa confidence intervals require more ",
+                                "(successful) bootstrap runs (", nrow(BOOT),
+                                ") than the number of observations (",
+                                ntotal, ").", sep = "")
+                   stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+               }
+
+               # does not work with sampling weights (yet)
+               if(!is.null(object@Data@weights[[1]])) {
+                   stop("lavaan ERROR: BCa confidence intervals not available in the presence of sampling weights.")
+               }
+
+               # check if we have a seed
+               if(is.null(bootstrap.seed)) {
+                   stop("lavaan ERROR: seed not available in BOOT object.")
+               }
+
+               # compute 'X' matrix with frequency indices (to compute
+               # the empirical influence values using regression)
+               FREQ <- lav_utils_bootstrap_indices(R = lavoptions$bootstrap,
+                   nobs = nobs, parallel = lavoptions$parallel[1],
+                   ncpus = lavoptions$ncpus, cl = lavoptions[["cl"]],
+                   iseed = bootstrap.seed, return.freq = TRUE,
+                   merge.groups = TRUE)
+               if(length(error.idx) > 0L) {
+                   FREQ <- FREQ[-error.idx, , drop = FALSE]
+               }
+               stopifnot(nrow(FREQ) == nrow(BOOT))
+
+               # compute empirical influence values (using regression)
+               # remove first column per group
+               first.idx <- sapply(object@Data@case.idx, "[[", 1L)
+               LM <- lm.fit(x = cbind(1, FREQ[,-first.idx]), y = BOOT)
+               BETA <- unname(LM$coefficients)[-1,,drop = FALSE]
+               LL <- rbind(0, BETA)
+
+               # compute 'a' for all parameters at once
+               AA <- apply(LL, 2L, function(x) {
+                           L <- x - mean(x); sum(L^3)/(6*sum(L^2)^1.5) })
+
+               # adjustment for both bias AND scale
+               alpha <- (1 + c(-level, level))/2
+               zalpha <- qnorm(alpha)
+               ci <- cbind(LIST$est, LIST$est)
+
+               # free.idx only
+               free.idx <- which(object@ParTable$free &
+                                 !duplicated(object@ParTable$free))
+               stopifnot(length(free.idx) == ncol(BOOT))
+               x <- LIST$est[free.idx]
+               for(i in 1:length(free.idx)) {
+                   t <- BOOT[,i]; t <- t[is.finite(t)]; t0 <- x[i]
+                   # check if we have variance (perhaps constrained to 0?)
+                   # new in 0.6-3
+                   if(var(t) == 0) {
+                       next
+                   }
+                   w <- qnorm(sum(t < t0)/length(t))
+                   a <- AA[i]
+                   adj.alpha <- pnorm(w + (w + zalpha)/(1 - a*(w + zalpha)))
+                   qq <- norm.inter(t, adj.alpha)
+                   ci[free.idx[i],] <- qq[,2]
+               }
+
+               # def.idx
+               def.idx <- which(object@ParTable$op == ":=")
+               if(length(def.idx) > 0L) {
+                   x.def <- object@Model@def.function(x)
+                   BOOT.def <- apply(BOOT, 1, object@Model@def.function)
+                   if(length(def.idx) == 1L) {
+                       BOOT.def <- as.matrix(BOOT.def)
+                   } else {
+                       BOOT.def <- t(BOOT.def)
+                   }
+
+                   # recompute empirical influence values
+                   LM <- lm.fit(x = cbind(1, FREQ[,-1]), y = BOOT.def)
+                   BETA <- unname(LM$coefficients)[-1,,drop = FALSE]
+                   LL <- rbind(0, BETA)
+
+                   # compute 'a' values for all def.idx parameters
+                   AA <- apply(LL, 2L, function(x) {
+                       L <- x - mean(x); sum(L^3)/(6*sum(L^2)^1.5) })
+
+                   # compute bca ci
+                   for(i in 1:length(def.idx)) {
+                       t <- BOOT.def[,i]; t <- t[is.finite(t)]; t0 <- x.def[i]
+                       w <- qnorm(sum(t < t0)/length(t))
+                       a <- AA[i]
+                       adj.alpha <- pnorm(w + (w + zalpha)/(1 - a*(w + zalpha)))
+                       qq <- norm.inter(t, adj.alpha)
+                       ci[def.idx[i],] <- qq[,2]
+                   }
+               }
+
                # TODO:
                # - add cin/ceq
             }
@@ -637,15 +742,32 @@ parameterestimates <- function(object,
             warning("lavaan WARNING: rsquare = TRUE, but there are no dependent variables")
         } else {
             if(lav_partable_nlevels(LIST) == 1L) {
-            R2 <- data.frame( lhs = NAMES, op = rep("r2", nel), rhs = NAMES,
-                              block = rep(1:length(r2), sapply(r2, length)),
-                              est = unlist(r2), stringsAsFactors = FALSE )
+                block <- rep(1:length(r2), sapply(r2, length))
+                first.block.idx <- which(!duplicated(LIST$block) &
+                                         LIST$block > 0L)
+                GVAL <- LIST$group[first.block.idx]
+                if(length(GVAL) > 0L) {
+                    group <- rep(GVAL, sapply(r2, length))
+                } else {
+                    # single block, single group
+                    group <- rep(1L, length(block))
+                }
+                R2 <- data.frame( lhs = NAMES, op = rep("r2", nel), rhs = NAMES,
+                                  block = block, group = group,
+                                  est = unlist(r2), stringsAsFactors = FALSE )
             } else {
-            # add level column
-            R2 <- data.frame( lhs = NAMES, op = rep("r2", nel), rhs = NAMES,
-                              block = rep(1:length(r2), sapply(r2, length)),
-                              level = rep(lav_partable_level_values(LIST),
-                                          sapply(r2, length)),
+                # add level column
+                block <- rep(1:length(r2), sapply(r2, length))
+                first.block.idx <- which(!duplicated(LIST$block) &
+                                         LIST$block > 0L)
+                # always at least two blocks
+                GVAL <- LIST$group[first.block.idx]
+                group <- rep(GVAL, sapply(r2, length))
+                LVAL <- LIST$level[first.block.idx]
+                level <- rep(LVAL, sapply(r2, length))
+                R2 <- data.frame( lhs = NAMES, op = rep("r2", nel), rhs = NAMES,
+                              block = block, group = group,
+                              level = level,
                               est = unlist(r2), stringsAsFactors = FALSE )
             }
             LIST <- lav_partable_merge(pt1 = LIST, pt2 = R2, warn = FALSE)
@@ -779,6 +901,14 @@ parameterestimates <- function(object,
         LIST$step <- NULL
     }
 
+    # always remove 'da' entries (if any)
+    if(any(LIST$op == "da")) {
+        da.idx <- which(LIST$op == "da")
+        LIST <- LIST[-da.idx,,drop = FALSE]
+    }
+
+
+
     # remove LIST$user
     LIST$user <- NULL
 
