@@ -10,6 +10,7 @@
 lav_start <- function(start.method    = "default",
                       lavpartable     = NULL,
                       lavsamplestats  = NULL,
+                      lavh1           = NULL, # fixme: only use lavh1?
                       model.type      = "sem",
                       mimic           = "lavaan",
                       reflect         = FALSE,    # rotation only
@@ -22,6 +23,9 @@ lav_start <- function(start.method    = "default",
     # categorical?
     categorical <- any(lavpartable$op == "|")
 
+    # correlation structure?
+    correlation <- any(lavpartable$op == "~*~")
+
     # conditional.x?
     conditional.x <- any(lavpartable$exo == 1L &
                          lavpartable$op %in% c("~", "<~"))
@@ -29,6 +33,7 @@ lav_start <- function(start.method    = "default",
 
     # nlevels?
     nlevels <- lav_partable_nlevels(lavpartable)
+    nblocks <- lav_partable_nblocks(lavpartable)
 
     # reflect/order.lv.by
     if(is.null(reflect)) {
@@ -45,7 +50,7 @@ lav_start <- function(start.method    = "default",
     } else if(mimic == "Mplus") {
         start.initial <- "mplus"
     } else {
-        # FIXME: use LISREL/EQS/AMOS/.... schems
+        # FIXME: use LISREL/EQS/AMOS/.... schemes
         start.initial <- "lavaan"
     }
 
@@ -57,7 +62,11 @@ lav_start <- function(start.method    = "default",
             # nothing to do
         } else if(start.method == "simple") {
             start <- numeric( length(lavpartable$ustart) )
-            start[ which(lavpartable$op == "=~") ] <- 1.0
+            if(categorical || correlation) {
+                start[ which(lavpartable$op == "=~") ] <- 0.7
+            } else {
+                start[ which(lavpartable$op == "=~") ] <- 1.0
+            }
             start[ which(lavpartable$op == "~*~") ] <- 1.0
             ov.names.ord <- vnames(lavpartable, "ov.ord")
             var.idx <- which(lavpartable$op == "~~" &
@@ -66,7 +75,7 @@ lav_start <- function(start.method    = "default",
             start[var.idx] <- 1.0
             user.idx <- which(!is.na(lavpartable$ustart))
             start[user.idx] <- lavpartable$ustart[user.idx]
-            return(start)
+            return(start) # assuming fixed.x = FALSE!
         } else if(start.method == "est") {
             return(lavpartable$est)
         } else if(start.method. %in% c("simple", "lavaan", "mplus")) {
@@ -112,7 +121,7 @@ lav_start <- function(start.method    = "default",
     start <- numeric( length(lavpartable$ustart) )
 
     # 1. =~ factor loadings:
-    if(categorical) {
+    if(categorical || correlation) {
         # if std.lv=TRUE, 0.8 is too large
         start[ which(lavpartable$op == "=~") ] <- 0.7
     } else {
@@ -149,7 +158,7 @@ lav_start <- function(start.method    = "default",
         group.values <- lav_partable_group_values(lavpartable)
 
         # info from user model for this group
-        if(conditional.x && nlevels == 1L) {
+        if(conditional.x) {
             ov.names     <- vnames(lavpartable, "ov.nox", group = group.values[g])
         } else {
             ov.names     <- vnames(lavpartable, "ov", group = group.values[g])
@@ -225,8 +234,8 @@ lav_start <- function(start.method    = "default",
                                    lavpartable$group == group.values[g] &
                                    lavpartable$rhs == f)
                 if(length(var.f.idx) > 0L &&
-                   lavpartable$free[var.f.idx] == 0 &&
-                   lavpartable$ustart[var.f.idx] == 1) {
+                   all(lavpartable$free[var.f.idx] == 0) &&
+                   all(lavpartable$ustart[var.f.idx] == 1)) {
                     std.lv <- TRUE
                 }
 
@@ -260,7 +269,11 @@ lav_start <- function(start.method    = "default",
 
                     # check for negative triad if nvar=3L (new in 0.6-8)
                     if(!is.null(fabin$neg.triad) && fabin$neg.triad) {
-                        tmp <- rep(1.0, length(tmp))
+                        if(std.lv) {
+                            tmp <- rep(0.7, length(tmp))
+                        } else {
+                            tmp <- rep(1.0, length(tmp))
+                        }
                     }
                     start[lambda.idx] <- tmp
 
@@ -334,7 +347,7 @@ lav_start <- function(start.method    = "default",
                             COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
                                                       ov.idx, drop = FALSE]
                         } else {
-                            if(conditional.x && nlevels == 1L) {
+                            if(conditional.x) {
                                 COV <- lavsamplestats@res.cov[[g]][ov.idx,
                                                           ov.idx, drop = FALSE]
                             } else {
@@ -512,7 +525,7 @@ lav_start <- function(start.method    = "default",
             reg.idx <- which(lavpartable$group == group.values[g] &
                              lavpartable$op == "~")
             if(length(reg.idx) > 0L) {
-                eqs.y <- unique(lavpartable$lhs[reg.idx])        
+                eqs.y <- unique(lavpartable$lhs[reg.idx])
                 ny <- length(eqs.y)
                 for(i in seq_len(ny)) {
                     y.name <- eqs.y[i]
@@ -520,7 +533,7 @@ lav_start <- function(start.method    = "default",
                                        lavpartable$op == "~" &
                                        lavpartable$lhs == y.name)
                     x.names <- lavpartable$rhs[start.idx]
-                    COV <- lavsamplestats@cov[[g]] 
+                    COV <- lavsamplestats@cov[[g]]
                     y.idx <- match(y.name,  ov.names)
                     x.idx <- match(x.names, ov.names)
                     S.xx <- COV[x.idx, x.idx, drop = FALSE]
@@ -528,7 +541,7 @@ lav_start <- function(start.method    = "default",
                     # regression coefficient(s)
                     beta.i <- try(solve(S.xx, S.xy), silent = TRUE)
                     if(inherits(beta.i, "try-error")) {
-                        start[start.idx] <- rep(0, length(start.idx))
+                        start[start.idx] <- beta.i <- rep(0, length(start.idx))
                     } else {
                         start[start.idx] <- drop(beta.i)
                     }
@@ -538,7 +551,7 @@ lav_start <- function(start.method    = "default",
                                      lavpartable$lhs == y.name &
                                      lavpartable$rhs == y.name)
                     res.val <- COV[y.idx, y.idx] - drop(crossprod(beta.i, S.xy))
-                    if(res.val > 0.01*COV[y.idx, y.idx] && 
+                    if(res.val > 0.01*COV[y.idx, y.idx] &&
                        res.val < 0.99*COV[y.idx, y.idx]) {
                         start[res.idx] <- res.val
                     } else {
@@ -547,7 +560,7 @@ lav_start <- function(start.method    = "default",
                 }
             }
         }
-     
+
       #  # 8 latent variances (new in 0.6-2)
       #  lv.names.y <- vnames(lavpartable, "lv.y", group = group.values[g])
       #  lv.names.x <- vnames(lavpartable, "lv.x", group = group.values[g])
@@ -619,22 +632,44 @@ lav_start <- function(start.method    = "default",
         # nlevels > 1L
         if(nlevels > 1L) {
             level.values <- lav_partable_level_values(lavpartable)
-
-            if(!conditional.x && length(ov.names.x) > 0) {
+            # Note: ov.names.x contains all levels within a group!
+            if(length(ov.names.x) > 0) {
                 for(l in 1:nlevels) {
+                    # block number
+                    block <- (g - 1L)*nlevels + l
+
+                    this.block.x <- lav_partable_vnames(lavpartable, "ov.x",
+                                                  block = block)
+                    this.block.ov <- lav_partable_vnames(lavpartable, "ov",
+                                                         block = block)
+                    if(length(this.block.x) == 0L) {
+                        next
+                    }
+
                     # var/cov
                     exo.idx <- which(lavpartable$group == group.values[g] &
                                      lavpartable$level == level.values[l] &
                                      lavpartable$op == "~~"               &
-                                     lavpartable$lhs %in% ov.names.x      &
-                                     lavpartable$rhs %in% ov.names.x)
-                    row.idx <- match(lavpartable$lhs[exo.idx], ov.names)
-                    col.idx <- match(lavpartable$rhs[exo.idx], ov.names)
+                                     lavpartable$lhs %in% this.block.x    &
+                                     lavpartable$rhs %in% this.block.x)
 
-                    if(l == 1L) {
-                        COV <- lavsamplestats@YLp[[g]][[2]]$S.PW.start
+                    if(is.null(lavh1$implied$cov[[1]])) {
+                        row.idx <- match(lavpartable$lhs[exo.idx], ov.names)
+                        col.idx <- match(lavpartable$rhs[exo.idx], ov.names)
+                        if(l == 1L) {
+                            COV <- lavsamplestats@YLp[[g]][[2]]$S.PW.start
+                        } else {
+                            COV <- lavsamplestats@YLp[[g]][[l]]$Sigma.B
+                        }
                     } else {
-                        COV <- lavsamplestats@YLp[[g]][[l]]$Sigma.B
+                        row.idx <- match(lavpartable$lhs[exo.idx],this.block.ov)
+                        col.idx <- match(lavpartable$rhs[exo.idx],this.block.ov)
+                        COV <- lavh1$implied$cov[[block]]
+                    }
+                    # make sure starting values for variances are positive
+                    neg.idx <- which(diag(COV) < 0.001)
+                    if(length(neg.idx) > 0L) {
+                        diag(COV)[neg.idx] <- 0.001
                     }
                     start[exo.idx] <- COV[ cbind(row.idx, col.idx) ]
 
@@ -642,16 +677,90 @@ lav_start <- function(start.method    = "default",
                     ov.int.idx <- which(lavpartable$group == group.values[g] &
                                         lavpartable$level == level.values[l] &
                                         lavpartable$op == "~1"               &
-                                        lavpartable$lhs %in% ov.names.x)
-                    idx <- match(lavpartable$lhs[ov.int.idx], ov.names)
+                                        lavpartable$lhs %in% this.block.x)
 
-                    if(l == 1L) {
-                        INT <- lavsamplestats@YLp[[g]][[2]]$Mu.W
+                    if(is.null(lavh1$implied$mean[[1]])) {
+                        idx <- match(lavpartable$lhs[ov.int.idx], ov.names)
+                        if(l == 1L) {
+                            INT <- lavsamplestats@YLp[[g]][[2]]$Mu.W
+                        } else {
+                            INT <- lavsamplestats@YLp[[g]][[l]]$Mu.B.start
+                        }
                     } else {
-                        INT <- lavsamplestats@YLp[[g]][[l]]$Mu.B.start
+                        idx <- match(lavpartable$lhs[ov.int.idx], this.block.ov)
+                        INT <- lavh1$implied$mean[[block]]
                     }
-
                     start[ov.int.idx] <- INT[idx]
+
+                    # new in 0.6-12
+                    # very special case: conditional.x with a combination of
+                    # splitted-x and regular-x
+                    # here, we must:
+                    # 1) replace var/cov of splitted-x by *residual* varcov
+                    #    after regressing out regular-x
+                    # 2) replace means of splitted-x by intercepts
+                    # 3) fill splitted-x ~ regular-x regression coefficients
+                    if(conditional.x) {
+                        if(is.null(lavh1$implied$cov[[l]])) {
+                            stop("lavaan ERROR: lavh1 information is needed; please rerun with h1 = TRUE")
+                        }
+                        blocks.within.group <- (g - 1L) * nlevels + seq_len(nlevels)
+                        OTHER.BLOCK.NAMES <- lav_partable_vnames(lavpartable, "ov",  block = blocks.within.group[-block])
+                        ov.names.x.block <- this.block.x
+                        idx <- which(ov.names.x.block %in% OTHER.BLOCK.NAMES)
+                        if(length(idx) > 0L) {
+                            ov.names.x.block <- ov.names.x.block[-idx]
+                        }
+                        ov.names.x1 <- this.block.x[!this.block.x %in% ov.names.x.block]
+                        ov.names.x2 <- ov.names.x.block
+                        nx1 <- length(ov.names.x1) # splitted x
+                        nx2 <- length(ov.names.x2) # regular  x
+                        if(nx1 > 0L && nx2 > 0L) {
+                            # COV
+                            c1.idx <- match(ov.names.x1, this.block.ov)
+                            c2.idx <- match(ov.names.x2, this.block.ov)
+
+                            COV.Y  <- COV[c1.idx, c1.idx, drop = FALSE]
+                            COV.X  <- COV[c2.idx, c2.idx, drop = FALSE]
+                            COV.YX <- COV[c1.idx, c2.idx, drop = FALSE]
+                            COV.XY <- COV[c2.idx, c1.idx, drop = FALSE]
+                            COV.XinvYX <- solve(COV.X, COV.XY)
+                            RES.COV <- COV.Y - COV.YX %*% COV.XinvYX
+
+                            res.cov.idx <- which(lavpartable$group == group.values[g] &
+                                     lavpartable$level == level.values[l] &
+                                     lavpartable$op == "~~"               &
+                                     lavpartable$lhs %in% ov.names.x1     &
+                                     lavpartable$rhs %in% ov.names.x1)
+
+                            row.idx <- match(lavpartable$lhs[res.cov.idx],ov.names.x1)
+                            col.idx <- match(lavpartable$rhs[res.cov.idx],ov.names.x1)
+                            start[res.cov.idx] <- RES.COV[ cbind(row.idx, col.idx) ]
+
+                            # INT
+                            INT.Y <- INT[c1.idx]
+                            INT.X <- INT[c2.idx]
+                            RES.INT <- INT.Y - t(COV.XinvYX) %*% INT.X
+
+                            res.int.idx <- which(lavpartable$group == group.values[g] &
+                                        lavpartable$level == level.values[l] &
+                                        lavpartable$op == "~1"               &
+                                        lavpartable$lhs %in% ov.names.x1)
+                            idx <- match(lavpartable$lhs[res.int.idx], ov.names.x1)
+                            start[res.int.idx] <- RES.INT[idx]
+
+                            # REG
+                            reg.idx <- which(lavpartable$group == group.values[g] &
+                                     lavpartable$level == level.values[l] &
+                                     lavpartable$op == "~"                &
+                                     lavpartable$lhs %in% ov.names.x1     &
+                                     lavpartable$rhs %in% ov.names.x2)
+                            row.idx <- match(lavpartable$lhs[reg.idx],ov.names.x1)
+                            col.idx <- match(lavpartable$rhs[reg.idx],ov.names.x2)
+                            start[reg.idx] <- t(COV.XinvYX)[ cbind(row.idx, col.idx) ]
+                        } # special case
+
+                    } # conditional.x
                 } # levels
             } # fixed.x
         } # nlevels > 1L
@@ -690,7 +799,6 @@ lav_start <- function(start.method    = "default",
         #                    lavpartable$lhs %in% lv.names)
     }
 
-
     # adjust if outside bounds -- new in 0.6-6
     if(!is.null(lavpartable$lower)) {
         bad.idx <- which(start < lavpartable$lower)
@@ -706,6 +814,13 @@ lav_start <- function(start.method    = "default",
     }
 
 
+    # override if the model syntax contains explicit starting values (free only)
+    #user.idx <- which(!is.na(lavpartable$ustart) &
+    #                  lavpartable$user != 7L) # new in 0.6-7, if rotation and
+    #                                          # and we change the order of lv's
+    user.idx <- which(!is.na(lavpartable$ustart) & lavpartable$free > 0L)
+    start[user.idx] <- lavpartable$ustart[user.idx]
+
     # override if a user list with starting values is provided
     # we only look at the 'est' column for now
     if(!is.null(start.user)) {
@@ -736,11 +851,8 @@ lav_start <- function(start.method    = "default",
         }
     }
 
-    # override if the model syntax contains explicit starting values
-    #user.idx <- which(!is.na(lavpartable$ustart) &
-    #                  lavpartable$user != 7L) # new in 0.6-7, if rotation and
-    #                                          # and we change the order of lv's
-    user.idx <- which(!is.na(lavpartable$ustart))
+    # override fixed values with ustart values
+    user.idx <- which(!is.na(lavpartable$ustart) & lavpartable$free == 0L)
     start[user.idx] <- lavpartable$ustart[user.idx]
 
     # final check: no NaN or other non-finite values
