@@ -7,6 +7,7 @@
 # YR 25/02/2012: changed data slot (from list() to S4); data@X contains data
 
 # YR 26 Jan 2017: use '...' to capture the never-ending list of options
+# YR 07 Feb 2023: add ov.order= argument
 
 lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                    model              = NULL,
@@ -38,6 +39,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                    WLS.V              = NULL,
                    NACOV              = NULL,
 
+                   # internal order of ov.names
+                   ov.order           = "model",
+
                    # full slots from previous fits
                    slotOptions        = NULL,
                    slotParTable       = NULL,
@@ -73,12 +77,63 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         if(inherits(data, "data.frame")) {
             # just in case it is not a traditional data.frame
             data <- as.data.frame(data)
+
+        } else if (inherits(data, "lavMoments")) {
+          # This object must contain summary statistics
+          # e.g., created by lavaan.mi::poolSat
+
+          # set required-data arguments
+          if ("sample.cov" %in% names(data)) {
+            sample.cov  <- data$sample.cov
+          } else stop("When data= is of class lavMoments, it must contain sample.cov")
+
+          if ("sample.nobs" %in% names(data)) {
+            sample.nobs <- data$sample.nobs
+          } else stop("When data= is of class lavMoments, it must contain sample.nobs")
+
+          # check for optional-data arguments
+          if ("sample.mean" %in% names(data)) sample.mean <- data$sample.mean
+          if ("sample.th"   %in% names(data)) sample.th   <- data$sample.th
+          if ("NACOV"       %in% names(data)) NACOV       <- data$NACOV
+          if ("WLS.V"       %in% names(data)) WLS.V       <- data$WLS.V
+
+          # set other args not included in dotdotdot
+          if (length(data$lavOptions)) {
+            newdots <- setdiff(names(data$lavOptions), names(dotdotdot))
+            if (length(newdots)) {
+              for (dd in newdots) dotdotdot[[dd]] <- data$lavOptions[[dd]]
+            }
+          }
+
+          #FIXME: Should WLS.V be an I(dentity) matrix when ULS is requested?
+          #       Unused for point estimates, but still used to scale/shift test
+          # if (!is.null(dotdotdot$estimator)) {
+          #   if (grepl(pattern = "ULS", x = toupper(dotdotdot$estimator[1L]) ) &&
+          #       !is.null(WLS.V)) {
+          #     ## set to diagonal
+          #     if (is.list(WLS.V)) {
+          #       WLS.V <- lapply(WLS.V, function(w) {diag(w) <- 1 ; return(w) } )
+          #     } else diag(WLS.V) <- 1
+          #   }
+          # }
+
+          # get rid of data= argument
+          data <- NULL
         }
+
         if(is.function(data)) {
             stop("lavaan ERROR: data is a function; it should be a data.frame")
         }
     }
 
+    # new in 0.6-14: if NACOV and/or WLS.V are provided, we force
+    # ov.order="data" for now
+    # until we have reliable code to re-arrange/select col/rows for
+    # of NACOV/WLS.V based on the model-based ov.names
+    if(!is.null(NACOV) || !is.null(WLS.V)) {
+        ov.order <- "data"
+    }
+
     # backwards compatibility, control= argument (<0.5-23)
     if(!is.null(dotdotdot$control)) {
         # optim.method
@@ -185,6 +240,29 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         stop("lavaan ERROR: model is NULL!")
     }
 
+    # Ok, we got a flattened model; usually this a FLAT object, but it could
+    # also be an already lavaanified parTable, or a bare-minimum list with
+    # lhs/op/rhs/free elements
+
+    # new in 0.6-14
+    # if ov.order = "data", it would seem we need to intervene here;
+    # we do this by 'injecting' dummy lhs da rhs statement in FLAT, to
+    # 'trick' lav_partable_vnames() (which only sees the model!)
+    ov.order <- tolower(ov.order)
+    if(ov.order == "data") {
+        FLAT.orig <- FLAT
+        try(FLAT <- lav_partable_ov_from_data(FLAT, data = data,
+                                              sample.cov = sample.cov,
+                                              slotData   = slotData),
+            silent = TRUE)
+        if(inherits(FLAT, "try-error")) {
+            warning("lavaan WARNING: ov.order = \"data\" setting failed; switching back to ov.order = \"model\"")
+            FLAT <- FLAT.orig
+        }
+    } else if(ov.order != "model") {
+        stop("lavaan ERROR: ov.order= argument should be \"model\" (default) or \"data\"")
+    }
+
     # group blocks?
     if(any(FLAT$op == ":" & tolower(FLAT$lhs) == "group")) {
         # here, we only need to figure out:
@@ -473,10 +551,17 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         opt <- modifyList(opt, dotdotdot)
 
         # no data?
-        if(is.null(data) && is.null(sample.cov)) {
+        if(is.null(slotData) && is.null(data) && is.null(sample.cov)) {
             opt$bounds <- FALSE
         }
 
+        # only sample moments?
+        if(!is.null(slotData) && !slotData@data.type == "full") {
+            opt$missing <- "listwise"
+        } else if(is.null(slotData) && is.null(data)) {
+            opt$missing <- "listwise"
+        }
+
         # categorical mode?
         opt$.categorical <- FALSE
         if(any(FLAT$op == "|")) {
@@ -502,6 +587,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                 opt$.categorical <- TRUE
             }
         }
+        if(tolower(opt$estimator) == "catml") {
+            opt$.categorical <- FALSE
+        }
 
         # clustered?
         if(length(cluster) > 0L) {
@@ -650,6 +738,12 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         }
         # more needed?
     }
+    # sanity check
+    if(!is.null(slotParTable) || inherits(model, "lavaan")) {
+        if(ngroups != lavdata@ngroups) {
+            stop("lavaan ERROR: mismatch between number of groups in data, and number of groups in model.")
+        }
+    }
     timing$Data <- (proc.time()[3] - start.time)
     start.time <- proc.time()[3]
     if(lavoptions$verbose) {
@@ -698,10 +792,18 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         #    tmp <- vnames(FLAT, type = "ov.x", ov.x.fatal = TRUE)
         #}
 
+        if(lavoptions$estimator == "catML") {
+            lavoptions$meanstructure <- FALSE
+            DataOV <- lavdata@ov
+            DataOV$type <- rep("numeric", length(DataOV$type))
+        } else {
+            DataOV <- lavdata@ov
+        }
+
         lavpartable <-
             lavaanify(model            = FLAT,
                       constraints      = constraints,
-                      varTable         = lavdata@ov,
+                      varTable         = DataOV,
                       ngroups          = lavdata@ngroups,
 
                       meanstructure    = lavoptions$meanstructure,
@@ -714,6 +816,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                       conditional.x    = lavoptions$conditional.x,
                       fixed.x          = lavoptions$fixed.x,
                       std.lv           = lavoptions$std.lv,
+                      correlation      = lavoptions$correlation,
                       effect.coding    = lavoptions$effect.coding,
                       ceq.simple       = lavoptions$ceq.simple,
                       parameterization = lavoptions$parameterization,
@@ -1316,8 +1419,17 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             cat("lavoptim           ... start:\n")
         }
 
+        # non-iterative methods (fabin, ...)
+        if(lavoptions$optim.method == "noniter") {
+            x <- try(lav_optim_noniter(lavmodel       = lavmodel,
+                                       lavsamplestats = lavsamplestats,
+                                       lavdata        = lavdata,
+                                       lavpartable    = lavpartable,
+                                       lavpta         = lavpta,
+                                       lavoptions     = lavoptions),
+                     silent = TRUE)
         # EM for multilevel models
-        if(lavoptions$optim.method == "em") {
+        } else if(lavoptions$optim.method == "em") {
             # multilevel only for now
             stopifnot(lavdata@nlevels > 1L)
             x <- try(lav_mvnorm_cluster_em_h0(lavsamplestats = lavsamplestats,
@@ -1334,11 +1446,6 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         # Gauss-Newton
         } else if(lavoptions$optim.method == "gn") {
             # only tested for DLS (for now)
-            #if(lavoptions$estimator != "DLS") {
-            #    stop("lavaan ERROR: optim.method = ", dQuote("gn"),
-            #         " is only available for estimator = ", dQuote("DLS"),
-            #         " (for now).")
-            #}
             x <- try(lav_optim_gn(lavmodel       = lavmodel,
                                   lavsamplestats = lavsamplestats,
                                   lavdata        = lavdata,
@@ -1938,7 +2045,19 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                   external     = list()               # empty list
                  )
 
-
+    # if model.type = "efa", add standardized solution to partable
+    if( (.hasSlot(lavmodel, "nefa")) && (lavmodel@nefa > 0L)) {
+        if(lavoptions$verbose) {
+            cat("computing standardized solution ... ")
+        }
+        STD <- standardizedSolution(lavaan, remove.eq = FALSE,
+                                    remove.ineq = FALSE, remove.def = FALSE)
+        if(lavoptions$verbose) {
+            cat(" done.\n")
+        }
+        lavaan@ParTable$est.std <- STD$est.std
+        lavaan@ParTable$se.std  <- STD$se
+    }
 
     # post-fitting check of parameters
     if(!is.null(lavoptions$check.post) && lavoptions$check.post &&
@@ -1988,6 +2107,9 @@ cfa <- sem <- function(# user-specified model: can be syntax, parameter Table
                        WLS.V              = NULL,
                        NACOV              = NULL,
 
+                       # internal order of ov.names
+                       ov.order           = "model",
+
                        # options (dotdotdot)
                        ...) {
 
@@ -2063,6 +2185,9 @@ growth <- function(# user-specified model: can be syntax, parameter Table
                    WLS.V              = NULL,
                    NACOV              = NULL,
 
+                   # internal order of ov.names
+                   ov.order           = "model",
+
                    # options (dotdotdot)
                    ...) {
 
